//==============================================================================
//
// Projet  : FICH2
//           Classes de gestion de fichiers (basees sur les flots d'E/S)
//
// Version : 1.00  Juillet 1999
// Auteur  : Denis ZIEGLER  denis.ziegler@worldonline.fr
//
// Fichier : FICH2.CPP
//
// Portabilite : C++ ANSI (DOS, Windows, Unix)
//
// (C) Copyright 1999 - ZIEGLER Denis
// Diffusion/modification autorisee selon les termes de la licence d'utilisation
//==============================================================================


#include <stdio.h>

#include "Fich2.h"



//===== Classe de base =========================================================

Fich::Fich(const char * _Nom)
{
  Nom[127] = 0;
  strncpy(Nom, _Nom, 127);
}

const char * Fich::getNom(void)
{
  return Nom;
}

void Fich::ChangeNom(const char *_Nom)
{
  strncpy(Nom, _Nom, 127);
}

int Fich::Creer(void)
{
  fs.clear();              //raz erreurs precedentes pour forcer l'E/S
  fs.open(Nom, ios::out);
  fs.close();
  return(fs.good());       //renvoi statut erreur
}

int Fich::Detruire(void)
{
  return(unlink(Nom)==0);
}

int Fich::Vider(void)
{
  return(Creer());
}

int Fich::Existe(void)
{
  fs.clear();              //raz erreurs precedentes pour forcer l'E/S
  fs.open(Nom, ios::in);
  fs.close();
  return(fs.good());       //renvoi statut erreur
}

int Fich::Ouvrir(const unsigned modeacces)
{
  int mod;

  switch (modeacces) {
    case MA_LECTURE :
      mod = ios::in | ios::binary | ios::nocreate;
      break;
    case MA_LECTECR :
      mod = ios::in | ios::out | ios::ate | ios::binary | ios::nocreate;
      break;
    case MA_ECRITURE :
      mod = ios::out | ios::ate | ios::binary;
      break;
    case MA_AJOUT :
      mod = ios::out | ios::app | ios::binary;
      break;
    case MA_ECRITRAZ :
      mod = ios::out | ios::binary;
      break;
    default: return(-1);
  }
  fs.clear();         //raz erreurs precedentes pour forcer l'E/S
  fs.open(Nom, mod);
  return(fs.good());  //renvoi statut erreur
}

int Fich::Fermer(void)
{
  fs.clear();         //raz erreurs precedentes pour forcer l'E/S
  fs.close();
  return(fs.good());  //renvoi statut erreur
}

int Fich::Lire(char *dest, const int nboctets)
{
  fs.clear();              //raz erreurs precedentes pour forcer l'E/S
  fs.read(dest, nboctets);
  return(fs.gcount());
}

int Fich::Ecrire(char *source, const int nboctets)
{
  fs.clear();         //raz erreurs precedentes pour forcer l'E/S
  fs.write(source, nboctets);
  return(fs.good());  //renvoi statut erreur
}

int Fich::AllerDebL(void)
{
  fs.clear();         //raz erreurs precedentes pour forcer l'E/S
  fs.seekg(0, ios::beg);
  return(fs.good());  //renvoi statut erreur
}

int Fich::AllerFinL(void)
{
  fs.clear();         //raz erreurs precedentes pour forcer l'E/S
  fs.seekg(0, ios::end);
  return(fs.good());  //renvoi statut erreur
}

int Fich::AllerPosL(const long n)
{
  fs.clear();         //raz erreurs precedentes pour forcer l'E/S
  fs.seekg(n);
  return(fs.good());  //renvoi statut erreur
}

int Fich::DecalerPosL(const long n)
{
  fs.clear();         //raz erreurs precedentes pour forcer l'E/S
  fs.seekg(n, ios::cur);
  return(fs.good());  //renvoi statut erreur
}

long Fich::PosCourL(void)
{
  return(fs.tellg());
}

int Fich::AllerDebE(void)
{
  fs.clear();         //raz erreurs precedentes pour forcer l'E/S
  fs.seekp(0, ios::beg);
  return(fs.good());  //renvoi statut erreur
}

int Fich::AllerFinE(void)
{
  fs.clear();         //raz erreurs precedentes pour forcer l'E/S
  fs.seekp(0, ios::end);
  return(fs.good());  //renvoi statut erreur
}

int Fich::AllerPosE(const long n)
{
  fs.clear();         //raz erreurs precedentes pour forcer l'E/S
  fs.seekp(n);
  return(fs.good());  //renvoi statut erreur
}

int Fich::DecalerPosE(const long n)
{
  fs.clear();         //raz erreurs precedentes pour forcer l'E/S
  fs.seekp(n, ios::cur);
  return(fs.good());  //renvoi statut erreur
}

long Fich::PosCourE(void)
{
  return(fs.tellp());
}

int Fich::Ok(void)
{
  return fs.good();
}

int Fich::Fin(void)
{
  return fs.eof();
}

int Fich::Erreur(void)
{
  return (fs.bad() || fs.fail());
}

long Fich::Taille(void)
{
  long p, rep;

  fs.clear();                //raz erreurs precedentes pour forcer l'E/S
  if ((p = PosCourL()) < 0)  //sauvegarde de la position courante
    return -1;               //sortie si erreur
  AllerFinL();
  rep = PosCourL();
  AllerPosL(p);              //restitution de la position precedente
  if (fs.good())
    return(rep);
  else
    return(-1);
}


//===== Fichier texte ==========================================================

int FichTxt::Ouvrir(const unsigned modeacces)
{
  int mod;

  switch (modeacces) {
    case MA_LECTURE :
      mod = ios::in | ios::nocreate;
      break;
    case MA_LECTECR :
      mod = ios::in | ios::out | ios::ate | ios::nocreate;
      break;
    case MA_ECRITURE :
      mod = ios::out | ios::ate;
      break;
    case MA_AJOUT :
      mod = ios::out | ios::app;
      break;
    case MA_ECRITRAZ :
      mod = ios::out;
      break;
    default: return(-1);
  }
  fs.clear();         //raz erreurs precedentes pour forcer l'E/S
  fs.open(Nom, mod);
  return(fs.good());  //renvoi statut erreur
}

int FichTxt::LireLgn(char *dest, const int lmax)
{
  fs.clear();         //raz erreurs precedentes pour forcer l'E/S
  fs.getline(dest, lmax);
  return(fs.good());  //renvoi statut erreur
}


int FichTxt::EcrireLgn(const char *source)
{
  fs.clear();         //raz erreurs precedentes pour forcer l'E/S
  fs << source << endl;
  return(fs.good());  //renvoi statut erreur
}

