//==============================================================================
//
// Projet  : FICH2
//           Classes de gestion de fichiers (basees sur les flots d'E/S)
//
// Version : 1.00  Juillet 1999
// Auteur  : Denis ZIEGLER  denis.ziegler@worldonline.fr
//
// Fichier : FICH2.H
//           Fichier inclus de FICH2.CPP
//
// Portabilite : C++ ANSI (DOS, Windows, Unix)
//
// (C) Copyright 1999 - ZIEGLER Denis
// Diffusion/modification autorisee selon les termes de la licence d'utilisation
//==============================================================================

#ifndef FICH2_H
#define FICH2_H

#include <fstream.h>


// Modes d'ouverture des fichiers
//   Positionnement en fin de fichier a la premiere ouverture, sauf mode lecture.
//   En modes ecriture seule (2,3,4) le fichier est cree s'il n'existe pas.
#define MA_LECTURE   0
#define MA_LECTECR   1
#define MA_ECRITURE  2
#define MA_AJOUT     3  //toujours ecriture en fin de fichier
#define MA_ECRITRAZ  4  //remise a zero du fichier a l'ouverture

// Taille maxi (par defaut) d'une ligne lue a partie d'un fichier texte
#define LMAXLGN 256


//===== Classe de base =========================================================

// Valeur de retour des fonctions (sauf indication contraire) :
//   1=Ok; 0=erreur

class Fich {
 protected :
   fstream fs;      //flot d'acces au fichier
   char Nom[128];   //nom complet du fichier (avec le chemin)
 public :
   Fich(const char *_Nom);    //constructeur
   const char * getNom(void); //renvoie le nom du fichier
   void ChangeNom(const char *_Nom);  //changement du nom du fichier
   int Creer(void);           //cree ou recree le fichier sur disque
   int Detruire(void);        //detruit le fichier sur le disque
   int Vider(void);           //remise a zero de la taille du fichier
   int Existe(void);          //teste si le fichier existe (1=oui; 0=non)
   virtual int Ouvrir(const unsigned ModeAcces); //ouvre le fichier
     //  ModeAcces : cf constantes MA_
     //  Valeur de retour : 1=Ok; 0=erreur; -1=modeacces invalide
   int Fermer(void); //Ferme le fichier
   int Lire(char *dest, const int nboctets);
     //Lit nboctets a la position courante
     // le pointeur de fichier avance de nboctets
     //  parametres : donnee destination, nombre d'octets a lire
     //  valeur de retour : nb d'octets lus (0=Erreur)
   int Ecrire(char *source, const int nboctets);
     //Ecrit nboctets a la position courante
     // le pointeur de fichier avance de nboctets
     //  parametres : donnee source, nombre d'octets a ecrire
   int AllerDebL(void);
     //Positionne le pointeur de lecture en debut de fichier
     // Attention ! Bien que les pointeurs de lecture et d'ecriture soient
     //   separes, avec des fonctions de manipulation separees, les mouvements
     //   de l'un entrainent generalement les memes mouvements de l'autre.
   int AllerFinL(void);
     //Positionne le pointeur de lecture en fin de fichier
   int AllerPosL(const long n);
     //Positionne le pointeur de lecture a la position n
   int DecalerPosL(const long n);
     //Decale la position du pointeur de lecture p/r a la position courante
   long PosCourL(void);
     //Donne la position courante du pointeur de lecture
     //  valeur de retour : -1=erreur; autre=position en octets
   int AllerDebE(void);  //memes fonctions avec le pointeur d'ecriture
   int AllerFinE(void);
   int AllerPosE(const long n);
   int DecalerPosE(const long n);
   long PosCourE(void);
   // Statut d'erreur apres la derniere operation d'E/S :
   int Ok(void);      //Teste si aucune condition d'erreur n'a ete rencontree
   int Fin(void);     //Teste si la fin de fichier a ete atteinte (1=oui 0=non)
   int Erreur(void);  //Teste si une erreur s'est produite (hors fin de fichier)
   long Taille(void);  //taille du fichier, en octets (-1=erreur)
};


//===== Fichier texte ==========================================================

class FichTxt : public Fich {
 public :
   FichTxt::FichTxt(const char * _Nom) : Fich(_Nom) { }
   virtual int Ouvrir(const unsigned ModeAcces); //ouvre le fichier
     // Meme syntaxe que pour la classe de base
     // La fonction est surchargee pour definir l'acces en mode texte
   int LireLgn(char *dest, const int lmax = LMAXLGN);
     //Lit une ligne dans le fichier a la position courante
     // le pointeur de fichier pointe sur la ligne suivante
     //  parametres : chaine destination,
     //               longueur maxi d'une ligne (y compris 0 de fin)
   int EcrireLgn(const char *source);
     //Ecrit une ligne a la position courante
     // le pointeur de fichier se positionne apres la fin de ligne
     //  parametres : donnee source
};


#endif